/* 
    3D Engine

    2005 Jason Hunt
    justinsanemsu@hotmail.com

    file: 3dmath.h
*/


#ifndef _3D_MATH_H
#define _3D_MATH_H

#include "types.h"

/* 
    3D operations
*/


point3d operator+(point3d lhs, point3d rhs);
point3d operator+=(point3d &lhs, point3d rhs);

point3d operator-(point3d lhs, point3d rhs);
point3d operator-=(point3d &lhs, point3d rhs);

point3d operator*=(point3d &lhs, double rhs);
point3d operator*(point3d lhs, double rhs);

point3d operator/=(point3d &lhs, double rhs);
point3d operator/(point3d lhs, double rhs);

void normalize(point3d &p);

point3d cross_product(point3d v1, point3d v2);

float dot_product(point3d a, point3d b);

float length(point3d v);

point3d get_normal(point3d p1, point3d p2, point3d p3);

point3d get_point(float x, float y, float z);

void update_trig(trig_type &trig, point3d &dir);

point3d abs(point3d p);

float calculate_light(point3d pos, point3d source, point3d normal);

point3d get_direction_vector( trig_type trig );


/* 
    2D operations
*/

point2d operator+(point2d lhs, point2d rhs);
point2d operator+=(point2d &lhs, point2d rhs);

point2d operator-(point2d lhs, point2d rhs);
point2d operator-=(point2d &lhs, point2d rhs);

point2d operator*=(point2d &lhs, double rhs);
point2d operator*(point2d lhs, double rhs);

point2d operator/=(point2d &lhs, double rhs);
point2d operator/(point2d lhs, double rhs);

float dot_product(point2d a, point2d b);

float length(point2d v);

point2d abs(point2d p);

point2d get_point(float x, float y);

float f_rand(float start, float end);

#endif

