/* 
    3D Engine

    2005 Jason Hunt
    justinsanemsu@hotmail.com

    file: camera.cpp
*/

#include <windows.h>
#include <stdio.h>
#include <math.h>

#include "world.h"
#include "camera.h"
#include "types.h"
#include "debug.h"
#include "3dmath.h"

extern world_class world;


/* init camera data */
void camera_class::start( void )
{
    position = get_point(0, 0, -3.0);
    direction = get_point(0, 0, 0);

    needs_update = true;

    update();
}
/* end of start */


/* update camera data */
void camera_class::update( void )
{
    if( !needs_update) return;

    POINT p;

    // get current mouse position
    GetCursorPos(&p);   

    // how far has the mouse moved?
    direction.y += mouse_sensitivity * (p.x - half_display_x) / (float)display_x;
    direction.x += mouse_sensitivity * (p.y - half_display_y) / (float)display_y;

    // reset mouse
    SetCursorPos(half_display_x, half_display_y);

    // clamp y angle
    if (direction.x > look_x_max) direction.x = look_x_max;
    if (direction.x < look_x_min) direction.x = look_x_min;

    // update trig data
    // needs loopup table
    update_trig(trig, direction);
    needs_update = false;
}
/* end of update */


/* move camera by a scaler */
void camera_class::move(float s)
{
    point3d d = get_direction_vector(trig);
    point3d n = d * s + position;

 /*   int r =  (world.check_area(n, 0.2));

    if (r != -1)
    {

        world.objects[r]->position += d*0.01;

    } else*/


  //  if (world.check_area(n, 0.12)) 
        position = n;
}
/* end of move camera */


/* move camera by a scaler */
void camera_class::strafe(float s)
{
   position.x += s * trig.cos_y;
   position.z -= s * trig.sin_y;
}
/* end of move camera */

