/* 
    3D Engine

    2005 Jason Hunt
    justinsanemsu@hotmail.com

    file: main.cpp
*/


#include <windows.h>
#include <stdio.h>

#include "3dmath.h"
#include "main.h"
#include "window.h"
#include "display.h"
#include "input.h"
#include "debug.h"
#include "camera.h"
#include "world.h"
#include "profile.h"
#include "texture.h"


extern unsigned int fps;
extern unsigned int frames;
extern world_class world;

HWND main_win = NULL;              // keep track of the main window
HDC main_dc= NULL;

bool paused = false;
bool shader = true;

/* ran when window is created */
void system_startup(HWND hwnd)
{
    reset_debug();

    init_profile(); 

    add_debug("system_startup", "Program start");

    input_start();

    ShowCursor(FALSE);
    SetCursorPos(half_display_x, half_display_y);
    
    create_buffer();    // setup off screen buffer

    world.start_engine();

    load_texture("tx1.bmp", 0);
    load_texture("tx2.bmp", 1);
    load_texture("tx3.bmp", 2);
    load_texture("tx4.bmp", 3);
    load_texture("tx5.bmp", 4);
    load_texture("tx6.bmp", 5);


//    world.add_cube(0.1, 4,false);

     world.add_sphere(10, false, 1); 

    world.objects[world.num_object-1]->shade = false;

    world.add_objects(15);

    SetTimer(hwnd, ID_TIME, 30, NULL);
    SetTimer(hwnd, ID_SEC, 1000, NULL);
}
/* end of system_startup */


/* ran when window is closed */
void system_shutdown(void)
{
    add_debug("system_shutdown", "Program stop");

    save_profile();

    char b[400];
    sprintf(b, "FPS: %d   Objects: %d  Total faces: %d  Visible faces: %d", fps, 
               world.num_object, world.total_triangle, world.num_visible_triangle);

    add_debug("", b);

    world.stop_engine();

    ShowCursor(TRUE);

    PostQuitMessage(0);   
}
/* end of system_shutdown */


/* deal with the system timer */
void timer_message(WPARAM wParam)
{
    if (wParam == ID_TIME) // display?
    {
        world.render_frame(main_win);
    } else
    if (wParam == ID_SEC) // update FPS?
    {
        fps = frames;
        frames = 0;
    }  
}
/* end of timer message */


/* the message handler */
LRESULT CALLBACK WinProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)                 
    {
        case WM_CREATE:         system_startup(hwnd);    break;
        case WM_TIMER:          timer_message(wParam); break;
        case WM_DESTROY:        system_shutdown(); break;
        case WM_KEYDOWN:        process_key_down(wParam); break;
        case WM_KEYUP:          process_key_up(wParam); break;
        case WM_PAINT:          display();  break;
        case WM_RBUTTONDOWN:    world.fire(); break;
        case WM_LBUTTONDOWN:    world.add_block(); break;
        case WM_MOUSEMOVE:      world.camera.needs_update = true; break;

        default: return DefWindowProc (hwnd, message, wParam, lParam);
    }

    return(0);
}
/* end of winproc */


/* winmain */
int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    // make the main window
    if (setup_window(hinstance, nCmdShow)) return(1);

    MSG messages;

    while (GetMessage (&messages, NULL, 0, 0))
    {
        TranslateMessage(&messages);
        DispatchMessage(&messages);
    }

    return(messages.wParam);
}
/* end of winmain */

