/* 
    3D Engine

    2005 Jason Hunt
    justinsanemsu@hotmail.com

    file: world.cpp
*/

#include <windows.h>
#include <malloc.h>
#include <math.h>
#include <stdio.h>

#include "profile.h"
#include "world.h"
#include "triangle.h"
#include "debug.h"
#include "display.h"
#include "3dmath.h"

world_class world; // contains all world data

double t = 0;
unsigned int z_buffer[display_x * display_y];

extern color_type buffer[display_x * display_y]; 
extern bool keys[256];

/* reset variables for engine start */
void world_class::start_engine( void )
{
    add_debug("world_class::start_engine", "starting engine");

    camera.start();

    // get memory for objects
    max_object = inital_max_objects;
    objects = (object_class **) malloc(max_object * sizeof(object_class *));

    light_source = get_point(0, 0, 0);

    color_type c;

    //make_textures();
}
/* end of start_engine */


/* shuedown engine and free buffers */
void world_class::stop_engine( void )
{
    add_debug("world_class::stop_engine", "stoping engine");

    for (unsigned int i = 0; i < num_object; i++)
    {
        objects[i]->unload();   // unload polygon and vertex data
        free(objects[i]);       // unload object
    }

    free(objects);              // unload object array
}
/* end of stop_engine */


/* check user input */
void world_class::check_input( void )
{
    if (keys['W']) camera.move(0.05);       // forward
    if (keys['S']) camera.move(-0.05);      // backward
    if (keys['A']) camera.strafe(-0.05);    // left
    if (keys['D']) camera.strafe(0.05);     // right

    if (keys['R']) camera.position.y += 0.05;    // left
    if (keys['F']) camera.position.y -= 0.05;     // right



    if (keys[VK_SPACE]) add_objects(5);

    if (keys['X']) 
    {
        add_sphere(10, true, rand()%num_tx);       
        objects[num_object-1]->randomize_placement();
    }

    if (keys['C']) 
    {
        add_cube(f_rand(0.01, 0.2), rand()%num_tx, true); 
        objects[num_object-1]->randomize_placement();
    }
}
/* end of check input */



/* insert a triangle pointer into the linked list */
void world_class::insert_triangle( triangle_class *t)
{
    triangle_class *tmp = visible_triangles;
    visible_triangles = t;
    t->next = tmp;
    
/*

    triangle_class *curr, *prev;

    curr = visible_triangles;
    prev = NULL;
  
//    while (curr && curr->average_z > t->average_z)
//    {
//        prev = curr;
//        curr = curr->next;
//    }

    t->next = curr;
   
    if (prev == NULL)
        visible_triangles = t;
    else
        prev->next = t;*/

    num_visible_triangle++;
}
/* end of insert triangle */


/* the code pipeline*/
void world_class::render_frame( HWND window )
{
     start_profile("render");

    // clear the display buffer to white 
    memset(buffer, 255, display_x * display_y * sizeof(color_type));

    // reset z buffer
    memset(z_buffer, 0, display_x * display_y * sizeof(unsigned int));

    if (num_object == 0) return;

    objects[0]->position.x = 1.0 * cos(t); 
    objects[0]->position.y = 1.0 * sin(t);

    t += 0.02;

    light_source.x = objects[0]->position.x;
    light_source.y = objects[0]->position.y;

    check_input();

    camera.update();

    num_visible_triangle = 0;
    total_triangle = 0;    

    visible_triangles = NULL;

    // transform each object
    for (unsigned int i = 0; i < num_object; i++)
    {
        object_class *obj = objects[i];

        start_profile("move");

        obj->move();

        stop_profile();

        start_profile("transform");

        obj->transform(camera); // transforminto world space

        stop_profile();

        start_profile("check visible");

        // copy triangles into buffer, if needed
        for (unsigned int tri = 0; tri < obj->num_triangle; tri++)
        {
            // add to render list if visible
            if (obj->triangle_data[tri].is_visible())
            {
                obj->triangle_data[tri].compute_values();
                insert_triangle(&obj->triangle_data[tri]);
            }

            total_triangle++; 
        }
        stop_profile();
    }

    // set pointer to start of list 
    triangle_class *cur_triangle = visible_triangles;


    start_profile("draw list");
    // draw the traingle list
    while (cur_triangle)
    {
        cur_triangle->draw();
        cur_triangle = cur_triangle->next;
    }

    stop_profile();

    // tell display to draw the buffer
    //InvalidateRect(main_win, NULL, FALSE); 

    start_profile("display");
    display();
    stop_profile();

    stop_profile();

}
/* end if render frame */


/* loads an object ito the world */
void world_class::load_object_from_file(char *filename)
{
    if (num_object >= max_object) reallocate_object_memory();

    objects[num_object] = (object_class*) malloc(sizeof(object_class));

    objects[num_object]->load_from_file(filename);

    num_object++;
}
/* end of load_object_from_file */






/* adds some objects to the the world */
void world_class::add_objects( unsigned int num )
{
    add_debug("world_class::add_objects", "");

//    for (int i = 0; i < num*2; i++)
//    {
//        add_cube(rand()%num_tx, true);
//        objects[num_object-1]->randomize_placement();
//    }


   for (int i = 0; i < num*2; i++)
   {
      add_cube(0.1, rand()%num_tx, true);
    // objects[num_object-1]->position.x = f_rand(-10, 10);   
  //   objects[num_object-1]->position.z = f_rand(-10, 10);   
//     objects[num_object-1]->position.y = f_rand(-0.05, 0.05);   

       objects[num_object-1]->randomize_placement();

}




   // for (int i = 0; i < num; i++)
   // {
  //      add_sphere(10, true);
  ///      objects[num_object-1]->randomize_placement();
  //  }
}
/* end if add objects */



/* double the avaialble memory */
void world_class::reallocate_object_memory( void )
{
    object_class **t;

    max_object *= 2;

    t = (object_class **) malloc(max_object * sizeof(object_class *));

    // copy data over
    for (int i = 0; i < num_object; i++)  t[i] = objects[i];

    free(objects); // release old memory 

    objects = t;    // copy list over;
}
/* end of reallocate_obect_memory */


void world_class::fire (void)
{
    add_cube(0.05, rand()%num_tx, true);

    point3d rate = get_direction_vector(camera.trig);

    world.objects[world.num_object - 1]->position = camera.position + rate * 0.5;

    world.objects[world.num_object - 1]->position_rate =  rate * 0.1;
}


void world_class::add_block (void)
{
    add_cube(0.1, rand()%num_tx, true);

    point3d rate = get_direction_vector(camera.trig);

//    world.objects[world.num_object - 1]->direction.x = camera.direction.x * -1;
//    world.objects[world.num_object - 1]->direction.y = camera.direction.y * -1;
//    world.objects[world.num_object - 1]->direction.z = camera.direction.z * -1;




    world.objects[world.num_object - 1]->position = camera.position + rate * 0.5;

//  update_trig(trig, direction);


//    world.objects[world.num_object - 1]->move();
//    world.objects[world.num_object - 1]->move();//
//    world.objects[world.num_object - 1]->move();
}





int world_class::check_area(point3d p, float size)
{
    for (int i = 0; i < world.num_object; i++)
      if (length(p - world.objects[i]->position) < world.objects[i]->radius + size) return(i);

    return(-1);
}

